from typing import Optional, Tuple

from dirlay import Dir
from pydantic_settings import BaseSettings
import pytest
from pytest import mark

from pydantic_file_secrets import (
    BaseSource,
    BuiltinSources,
    FileSecretsSettingsSource,
    SECRETS_DIR_MAX_SIZE,
    SettingsConfigDict,
    SettingsError,
    with_builtin_sources,
)


class Settings(BaseSettings):
    key1: Optional[str] = None
    key2: Optional[str] = None

    @classmethod
    @with_builtin_sources
    def settings_customise_sources(cls, src: BuiltinSources) -> Tuple[BaseSource, ...]:
        return (FileSecretsSettingsSource(src.file_secret_settings),)


@mark.parametrize(
    'conf,secrets,dirs,expected',
    (
        (
            # when multiple secrets_dir values are given, their values are merged
            dict(),
            Dir({'dir1/key1': 'a', 'dir1/key2': 'b', 'dir2/key2': 'c'}),
            ['dir1', 'dir2'],
            {'key1': 'a', 'key2': 'c'},
        ),
        (
            # when secrets_dir is not a directory, error is raised
            dict(),
            Dir({'some_file': ''}),
            'some_file',
            (SettingsError, 'must reference a directory'),
        ),
        (
            # missing secrets_dir emits warning by default
            dict(),
            Dir({'key1': 'value'}),
            'missing_subdir',
            (UserWarning, 1, 'does not exist', {'key1': None, 'key2': None}),
        ),
        (
            # ...or expect warning explicitly (identical behaviour)
            dict(secrets_dir_missing='warn'),
            Dir({'key1': 'value'}),
            'missing_subdir',
            (UserWarning, 1, 'does not exist', {'key1': None, 'key2': None}),
        ),
        (
            # missing secrets_dir warning can be suppressed
            dict(secrets_dir_missing='ok'),
            Dir({'key1': 'value'}),
            'missing_subdir',
            {'key1': None, 'key2': None},
        ),
        (
            # missing secrets_dir can raise error
            dict(secrets_dir_missing='error'),
            Dir({'key1': 'value'}),
            'missing_subdir',
            (SettingsError, 'does not exist'),
        ),
        (
            # invalid secrets_dir_missing value raises error
            dict(secrets_dir_missing='uNeXpEcTeD'),
            Dir({'key1': 'value'}),
            'missing_subdir',
            (SettingsError, 'invalid secrets_dir_missing value'),
        ),
        (
            # when multiple secrets_dir do not exist, multiple warnings are emitted
            dict(),
            Dir({'key1': 'value'}),
            ['missing_subdir1', 'missing_subdir2'],
            (UserWarning, 2, 'does not exist', {'key1': None, 'key2': None}),
        ),
        (
            # secrets_dir size is limited
            dict(),
            Dir({'key1': 'x' * SECRETS_DIR_MAX_SIZE}),
            '.',
            {'key1': 'x' * SECRETS_DIR_MAX_SIZE, 'key2': None},
        ),
        (
            # ...and raises error if file is larger than the limit
            dict(),
            Dir({'key1': 'x' * (SECRETS_DIR_MAX_SIZE + 1)}),
            '.',
            (SettingsError, 'secrets_dir size'),
        ),
        (
            # secrets_dir size limit can be adjusted
            dict(secrets_dir_max_size=100),
            Dir({'key1': 'x' * 100}),
            '.',
            {'key1': 'x' * 100, 'key2': None},
        ),
        (
            # ...and raises error if file is larger than the limit
            dict(secrets_dir_max_size=100),
            Dir({'key1': 'x' * 101}),
            '.',
            (SettingsError, 'secrets_dir size'),
        ),
        (
            # ...even if secrets_dir size exceeds limit because of another file
            dict(secrets_dir_max_size=100),
            Dir({'another_file': 'x' * 101}),
            '.',
            (SettingsError, 'secrets_dir size'),
        ),
        (
            # when multiple secrets_dir values are given, their sizes are not added
            dict(secrets_dir_max_size=100),
            Dir({'dir1/key1': 'x' * 100, 'dir2/key2': 'y' * 100}),
            ['dir1', 'dir2'],
            {'key1': 'x' * 100, 'key2': 'y' * 100},
        ),
    ),
)
def test_cases(conf: SettingsConfigDict, secrets, dirs, expected, tmp_path):
    secrets_dirs = (
        [tmp_path / d for d in dirs]
        if isinstance(dirs, list)
        else tmp_path / dirs
    )  # fmt: skip

    class MySettings(Settings):
        model_config = SettingsConfigDict(secrets_dir=secrets_dirs, **conf)

    with secrets.mktree(tmp_path):
        # clean execution
        if isinstance(expected, dict):
            assert MySettings().model_dump() == expected
        # error
        elif isinstance(expected, tuple) and len(expected) == 2:
            error_type, msg_fragment = expected
            with pytest.raises(error_type, match=msg_fragment):
                MySettings()
        # warnings
        elif isinstance(expected, tuple) and len(expected) == 4:
            warning_type, warning_count, msg_fragment, value = expected
            with pytest.warns(warning_type) as warninfo:
                settings = MySettings()
            assert len(warninfo) == warning_count
            assert all(msg_fragment in str(w.message) for w in warninfo)
            assert settings.model_dump() == value
        # unexpected
        else:
            raise AssertionError('unreachable')



from dirlay import Dir
from pytest import mark

from pydantic_file_secrets import SettingsConfigDict
from tests.sample import AppSettings


@mark.parametrize(
    'conf,secrets',
    (
        (
            dict(secrets_nested_delimiter='___', secrets_prefix='prefix_'),
            {'prefix_app_key': 'secret1', 'prefix_db___passwd': 'secret2'},
        ),
        (
            dict(secrets_nested_subdir=True, secrets_prefix='prefix_'),
            {'prefix_app_key': 'secret1', 'prefix_db/passwd': 'secret2'},
        ),
        (
            dict(secrets_nested_subdir=True, secrets_prefix='dir1/dir2/'),
            {'dir1/dir2/app_key': 'secret1', 'dir1/dir2/db/passwd': 'secret2'},
        ),
    ),
)
def test_prefix(conf: SettingsConfigDict, secrets, monkeypatch, tmp_path):
    monkeypatch.setenv('DB__USER', 'user')

    class Settings(AppSettings):
        model_config = SettingsConfigDict(
            env_nested_delimiter='__',
            secrets_dir=tmp_path,
            **conf,
        )

    with Dir(secrets).mktree(tmp_path):
        assert Settings().model_dump() == {
            'app_key': 'secret1',
            'db': {'user': 'user', 'passwd': 'secret2'},
        }



from collections.abc import Callable, Sequence
from functools import wraps
from pathlib import Path
from typing import NamedTuple, Tuple, Type, TypeVar, Union

from pydantic_settings import BaseSettings
from pydantic_settings.sources import PydanticBaseSettingsSource
from typing_extensions import TypeAlias

PathType: TypeAlias = Union[Path, str, Sequence[Union[Path, str]]]


# settings customise sources: syntactic sugar


T = TypeVar('T', bound=Type[BaseSettings])
BaseSource: TypeAlias = PydanticBaseSettingsSource


class BuiltinSources(NamedTuple):
    init_settings: PydanticBaseSettingsSource
    env_settings: PydanticBaseSettingsSource
    dotenv_settings: PydanticBaseSettingsSource
    file_secret_settings: PydanticBaseSettingsSource


def with_builtin_sources(
    method: Callable[[T, BuiltinSources], Tuple[BaseSource, ...]],
) -> Callable[
    [
        T,
        Type[BaseSettings],
        PydanticBaseSettingsSource,
        PydanticBaseSettingsSource,
        PydanticBaseSettingsSource,
        PydanticBaseSettingsSource,
    ],
    Tuple[BaseSource, ...],
]:
    @wraps(method)
    def simple(
        cls: T,
        setting_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[BaseSource, ...]:
        args = BuiltinSources(
            init_settings=init_settings,
            env_settings=env_settings,
            dotenv_settings=dotenv_settings,
            file_secret_settings=file_secret_settings,
        )
        return method(cls, args)

    return simple



from typing import Optional, Tuple

from pydantic import BaseModel
from pydantic_settings import BaseSettings

from pydantic_file_secrets import (
    BaseSource,
    BuiltinSources,
    FileSecretsSettingsSource,
    with_builtin_sources,
)


class DbSettings(BaseModel):
    user: str
    passwd: Optional[str] = None


class AppSettings(BaseSettings):
    app_key: Optional[str] = None
    db: DbSettings

    @classmethod
    @with_builtin_sources
    def settings_customise_sources(cls, src: BuiltinSources) -> Tuple[BaseSource, ...]:
        return (
            src.init_settings,
            src.env_settings,
            FileSecretsSettingsSource(src.file_secret_settings),
        )



from pydantic_file_secrets import FileSecretsSettingsSource
from pydantic_settings import BaseSettings, SecretsSettingsSource, SettingsConfigDict


def test_repr(tmp_path):
    class Settings(BaseSettings):
        model_config = SettingsConfigDict(
            secrets_dir=tmp_path,
        )

    src = FileSecretsSettingsSource(SecretsSettingsSource(Settings))
    assert f'{src!r}'.startswith(f'{src.__class__.__name__}(')



"""
These tests show that the interface of original SecretsSettingsSource declares features
that are not working:

    - env_ignore_empty
    - env_parse_none_str
    - env_parse_enums
    - str_strip_whitespace
"""

from enum import Enum
from typing import Optional

from dirlay import Dir
from pydantic_settings import BaseSettings
from pytest import mark

from pydantic_file_secrets import SettingsConfigDict


class SampleEnum(str, Enum):
    TEST = 'test'


class Settings(BaseSettings):
    some_str: Optional[str] = None
    some_enum: Optional[SampleEnum] = None


@mark.parametrize(
    'secrets,conf,expected',
    (
        ({'some_str': ''}, dict(env_ignore_empty=True), {'some_str': ''}),
        ({'some_str': 'null'}, dict(env_parse_none_str='null'), {'some_str': 'null'}),
        ({'some_enum': 'test'}, dict(env_parse_enums=False), {'some_enum': SampleEnum.TEST}),
        # whitespace is always stripped on secrets
        ({'some_str': 'value '}, dict(), {'some_str': 'value'}),
        ({'some_str': 'value '}, dict(str_strip_whitespace=False), {'some_str': 'value'}),
    ),
)  # fmt: skip
def test_not_working(secrets, conf: SettingsConfigDict, expected, tmp_path):
    class MySettings(Settings):
        model_config = SettingsConfigDict(secrets_dir=tmp_path, **conf)

    with Dir(secrets).mktree(tmp_path):
        settings = MySettings()
        for k, v in expected.items():
            assert getattr(settings, k) == v



from pydantic import BaseModel, SecretStr
from pydantic_file_secrets import FileSecretsSettingsSource, SettingsConfigDict
from pydantic_settings import BaseSettings
from pydantic_settings.sources import PydanticBaseSettingsSource


class DbSettings(BaseModel):
    passwd: SecretStr


class Settings(BaseSettings):
    app_key: SecretStr
    db: DbSettings

    model_config = SettingsConfigDict(
        secrets_dir='secrets',
        secrets_nested_delimiter='__',
    )

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> tuple[PydanticBaseSettingsSource, ...]:
        return (
            init_settings,
            env_settings,
            dotenv_settings,
            FileSecretsSettingsSource(file_secret_settings),
        )



from dirlay import Dir

from pydantic_file_secrets import SettingsConfigDict
from tests.sample import AppSettings


def test_strip_whitespace(monkeypatch, tmp_path):
    monkeypatch.setenv('DB__USER', 'user')
    secrets = Dir() | {
        'app_key': ' secret1 ',
        'db__passwd': '\tsecret2\n',
    }

    class Settings(AppSettings):
        model_config = SettingsConfigDict(
            env_nested_delimiter='__',
            secrets_dir=tmp_path,
            secrets_nested_delimiter='__',
        )

    with secrets.mktree(tmp_path):
        assert Settings(_secrets_dir=tmp_path).model_dump() == {
            'app_key': 'secret1',
            'db': {'user': 'user', 'passwd': 'secret2'},
        }



import os
import sys
from unittest import TestCase

from dirlay import Dir
from doctestcase import doctestcase


sys.path.insert(0, os.path.abspath('.'))

import tests.usage.multiple
import tests.usage.nested
import tests.usage.plain
import tests.usage.sugar


class SecretsManager:
    secrets_dir: Dir

    def setUp(self):
        self.secrets_dir.mktree(chdir=True)

    def tearDown(self):
        self.secrets_dir.rmtree()


@doctestcase(globals={'Settings': tests.usage.plain.Settings})
class UsagePlain(SecretsManager, TestCase):
    """
    Plain

    >>> Settings().model_dump()
    {'app_key': SecretStr('**********'), 'db': {'passwd': SecretStr('**********')}}
    """

    secrets_dir = Dir() | {
        'secrets': {
            'app_key': 'secret1',
            'db__passwd': 'secret2',
        },
    }


@doctestcase(globals={'Settings': tests.usage.nested.Settings})
class UsageNested(SecretsManager, TestCase):
    """
    Nested

    >>> Settings().model_dump()
    {'app_key': SecretStr('**********'), 'db': {'passwd': SecretStr('**********')}}
    """

    secrets_dir = Dir() | {
        'secrets': {
            'app_key': 'secret1',
            'db/passwd': 'secret2',
        },
    }


@doctestcase(globals={'Settings': tests.usage.multiple.Settings})
class UsageMultiple(SecretsManager, TestCase):
    """
    Multiple

    >>> Settings().model_dump()
    {'app_key': SecretStr('**********'), 'db': {'passwd': SecretStr('**********')}}
    """

    secrets_dir = Dir() | {
        'secrets': {
            'layer1/app_key': 'secret1',
            'layer2/db__passwd': 'secret2',
        },
    }


@doctestcase(globals={'Settings': tests.usage.sugar.Settings})
class UsageSugar(SecretsManager, TestCase):
    """
    Experimantal syntactic sugar

    >>> Settings().model_dump()
    {'app_key': SecretStr('**********'), 'db': {'passwd': SecretStr('**********')}}
    """

    secrets_dir = UsagePlain.secrets_dir



import json
import re
from dataclasses import dataclass
from pathlib import Path
from typing import Self
from unittest import TestCase

import bracex
from docsub import click
from doctestcase import to_markdown
from importloc import Location, get_subclasses, random_name
from tabulate import tabulate


@click.group()
def x() -> None:
    pass


@x.command()
@click.argument('testpath', type=click.Path(exists=True, dir_okay=False))
@click.argument('nameregex')
def cases(testpath: click.Path, nameregex: str) -> None:
    """
    Print usage section based on test case docstrings.
    """
    cases = get_subclasses(Location(str(testpath)).load(random_name), TestCase)
    cases.sort(key=lambda c: c.__firstlineno__)  # type: ignore[attr-defined]
    for case in cases:
        if re.fullmatch(nameregex, case.__name__):
            click.echo(to_markdown(case, include_title=False))


@x.command()
@click.argument('obj', type=str)
def dirtree(obj: str) -> None:
    """
    Print directory tree from Dir object.
    """
    tree = Location(obj).load(on_conflict='reuse')
    tree.print_rich(hide_root=True)


@dataclass
class TestResult:
    envname: str  # tox env name
    frag: tuple[str, ...]  # mapping of fragment prefixes to suffixes
    marks: set[str]  # pass marks

    @classmethod
    def from_tox_env(
        cls,
        envdir: Path,
        fragment_groups: list[list[str]],
        fragment_sep: str = '-',
        passmark_prefix: str = '.pass-',
    ) -> Self | None:
        # parse fragments
        fragments = set(envdir.name.split(fragment_sep))
        if len(fragments) != len(fragment_groups):
            return None
        frag = []
        for g in fragment_groups:
            match = fragments & set(g)
            if len(match) != 1:
                return None
            frag.append(match.pop())
        # load pass marks
        marks = set()
        for markfile in envdir.glob(f'{passmark_prefix}*'):
            marks.add(markfile.name.removeprefix(passmark_prefix))
        #
        return cls(envname=envdir.name, frag=tuple(frag), marks=marks)


@x.command()
@click.argument('toxdir', type=click.Path(file_okay=False, path_type=Path))
@click.option('-f', '--fragments', type=str)
@click.option('-i', '--icons', type=str)
def testres(toxdir: Path, fragments: str, icons: str) -> None:
    """
    Print matrix of tox test results.
    """
    if not toxdir.exists():
        return

    fgroup = [list(reversed(bracex.expand(g))) for g in fragments.split(';')]
    if len(fgroup) > 2:
        raise ValueError('Maximum 2 fragments are allowed')
    markicons = json.loads(icons)

    # load results
    results: list[TestResult] = []
    for toxenv in toxdir.iterdir():
        if not toxenv.is_dir():
            continue
        res = TestResult.from_tox_env(toxenv, fragment_groups=fgroup)
        if res is not None:
            results.append(res)

    # print matrix

    failicon = next(iter(markicons.values()))

    def result_icon(res: TestResult, default=failicon) -> str | None:
        for mark, icon in reversed(markicons.items()):
            if mark in res.marks:
                return icon
        return default

    headers = ['', *fgroup[1]]
    body = []
    for row in fgroup[0]:
        res = [r for r in results if r.frag[0] == row]
        res.sort(key=lambda r: fgroup[1].index(r.frag[1]))
        body.append([row, *(result_icon(r) for r in res)])
    table = tabulate(
        tabular_data=body,
        headers=headers,
        tablefmt='github',
        colalign=['left'] + ['center'] * len(fgroup[1]),
        maxcolwidths=[max(len(c) for c in fgroup[0]) + 2, *(len(c) for c in fgroup[1])],
    )

    click.echo(table)



from dirlay import Dir

from pydantic_file_secrets import SettingsConfigDict
from tests.sample import AppSettings


def test_subdir(monkeypatch, tmp_path):
    monkeypatch.setenv('DB__USER', 'user')
    # fmt: off
    secrets = Dir() | {
        'app_key': 'secret1',
        'db_random/passwd': 'secret2',  # file in subdir that is not directly referenced in our settings
    }
    # fmt: on

    class Settings(AppSettings):
        model_config = SettingsConfigDict(
            secrets_dir=tmp_path,
            env_nested_delimiter='__',
            secrets_nested_subdir=True,
        )

    with secrets.mktree(tmp_path):
        # create a symlink to the match our settings
        tmp_path.joinpath('db').symlink_to(tmp_path.joinpath('db_random'))

        assert Settings().model_dump() == {
            'app_key': 'secret1',
            'db': {'user': 'user', 'passwd': 'secret2'},
        }



"""
These tests show that nested secrets do not work in Pydantic Settings.
"""

from typing import Optional

from dirlay import Dir
from pydantic import BaseModel
from pydantic_settings import BaseSettings, SettingsConfigDict
import pytest
from pytest import mark


class DbSettings(BaseModel):
    user: str
    passwd: Optional[str] = None


class AppSettings(BaseSettings):
    app_key: Optional[str] = None
    db: DbSettings


@mark.parametrize(
    'secrets_dir,warning',
    (
        # file name matching nested field name does not work
        ({'passwd': 'secret1', 'app_key': 'secret2'}, None),
        # file name with env_nested_delimiter does not work
        ({'db__passwd': 'secret1', 'app_key': 'secret2'}, None),
        # subdirectory does not work
        ({'db/passwd': 'secret1', 'app_key': 'secret2'}, (UserWarning, 'found a directory instead')),
    ),
)  # fmt: skip
def test_failing_configuration(secrets_dir, warning, monkeypatch, tmp_path):
    monkeypatch.setenv('DB__USER', 'user')

    class Settings(AppSettings):
        model_config = SettingsConfigDict(
            secrets_dir=tmp_path,
            env_nested_delimiter='__',
        )

    with Dir(secrets_dir).mktree(tmp_path):
        if warning is None:
            settings = Settings()
        else:
            warning_type, msg_fragment = warning
            with pytest.warns(warning_type, match=msg_fragment):
                settings = Settings()

        assert settings.model_dump() == {
            'db': {'user': 'user', 'passwd': None},  # passwd is not loaded
            'app_key': 'secret2',  # app_key is loaded because it's not nested
        }



from dirlay import Dir
import pydantic_settings.main  # import to monkeypatch

from pydantic_file_secrets import FileSecretsSettingsSource


def test_dropin(monkeypatch, tmp_path):
    monkeypatch.setattr(
        pydantic_settings.main,
        'SecretsSettingsSource',
        FileSecretsSettingsSource,
    )
    secrets = Dir() | {
        'dir1/key1': 'secret1',
        'dir2/key2': 'secret2',
    }

    class Settings(pydantic_settings.BaseSettings):
        key1: str
        key2: str

    with secrets.mktree(tmp_path):
        conf = Settings(_secrets_dir=[secrets // 'dir1', secrets // 'dir2'])
        assert conf.model_dump() == {
            'key1': 'secret1',
            'key2': 'secret2',
        }



from collections.abc import Sequence
from pathlib import Path
from typing import Literal, Optional, Union

import pydantic_settings
from typing_extensions import TypeAlias

PathType: TypeAlias = Union[Path, str, Sequence[Union[Path, str]]]


class SettingsConfigDict(pydantic_settings.SettingsConfigDict, total=False):
    secrets_dir_missing: Optional[Literal['ok', 'warn', 'error']]
    secrets_dir_max_size: Optional[int]
    secrets_case_sensitive: Optional[bool]
    secrets_prefix: Optional[str]
    secrets_nested_delimiter: Optional[str]
    secrets_nested_subdir: Optional[bool]



from pydantic import BaseModel, SecretStr
from pydantic_file_secrets import (
    BaseSource,
    BuiltinSources,
    FileSecretsSettingsSource,
    SettingsConfigDict,
    with_builtin_sources,
)
from pydantic_settings import BaseSettings


class DbSettings(BaseModel):
    passwd: SecretStr


class Settings(BaseSettings):
    app_key: SecretStr
    db: DbSettings

    model_config = SettingsConfigDict(
        secrets_dir='secrets',
        secrets_nested_delimiter='__',
    )

    @classmethod
    @with_builtin_sources
    def settings_customise_sources(cls, src: BuiltinSources) -> tuple[BaseSource, ...]:
        return (
            src.init_settings,
            src.env_settings,
            src.dotenv_settings,
            FileSecretsSettingsSource(src.file_secret_settings),
        )



from enum import Enum
from typing import Optional

from dirlay import Dir
from pydantic_settings import BaseSettings
from pytest import mark

from pydantic_file_secrets import (
    BuiltinSources,
    FileSecretsSettingsSource,
    SettingsConfigDict,
    with_builtin_sources,
)


class SampleEnum(str, Enum):
    TEST = 'test'


class Settings(BaseSettings):
    field_empty: Optional[str] = None
    field_none: Optional[str] = None
    field_enum: Optional[SampleEnum] = None


@mark.parametrize(
    'conf,expected',
    (
        # default settings
        ({}, dict(field_empty='', field_none='null', field_enum=SampleEnum.TEST)),
        # env_ignore_empty has no effect on secrets
        ({'env_ignore_empty': True}, dict(field_empty='')),
        ({'env_ignore_empty': False}, dict(field_empty='')),
        # env_parse_none_str has no effect on secrets
        ({'env_parse_none_str': 'null'}, dict(field_none='null')),
        # env_parse_enums has no effect on secrets
        ({'env_parse_enums': True}, dict(field_enum=SampleEnum.TEST)),
        ({'env_parse_enums': False}, dict(field_enum=SampleEnum.TEST)),
    ),
)
def test_env_ignore_empty(conf: SettingsConfigDict, expected, tmp_path):
    secrets = Dir() | {
        'field_empty': '',
        'field_none': 'null',
        'field_enum': 'test',
    }

    class Original(Settings):
        model_config = SettingsConfigDict(secrets_dir=tmp_path, **conf)

    class Evaluated(Settings):
        model_config = SettingsConfigDict(secrets_dir=tmp_path, **conf)

        @classmethod
        @with_builtin_sources
        def settings_customise_sources(cls, src: BuiltinSources):
            return (FileSecretsSettingsSource(src.file_secret_settings),)

    with secrets.mktree(tmp_path):
        original = Original()
        evaluated = Evaluated()
        assert original.model_dump() == evaluated.model_dump()
        for k, v in expected.items():
            assert getattr(original, k) == getattr(evaluated, k) == v



from dirlay import Dir

from pydantic_file_secrets import SettingsConfigDict
from tests.sample import AppSettings


def test_delimited_name(monkeypatch, tmp_path):
    monkeypatch.setenv('DB__USER', 'user')
    secrets = Dir() | {
        'app_key': 'secret1',
        'db___passwd': 'secret2',
    }

    class Settings(AppSettings):
        model_config = SettingsConfigDict(
            env_nested_delimiter='__',
            secrets_dir=tmp_path,
            secrets_nested_delimiter='___',
        )

    with secrets.mktree(tmp_path):
        assert Settings().model_dump() == {
            'app_key': 'secret1',
            'db': {'user': 'user', 'passwd': 'secret2'},
        }


def test_secrets_off(monkeypatch, tmp_path):
    monkeypatch.setenv('DB__USER', 'user')
    secrets = Dir() | {
        'app_key': 'secret1',
        'db__passwd': 'secret2',
    }

    class Settings(AppSettings):
        model_config = SettingsConfigDict(
            env_nested_delimiter='__',
        )

    with secrets.mktree(tmp_path):
        assert Settings().model_dump() == {
            'app_key': None,
            'db': {'user': 'user', 'passwd': None},
        }
